@@ -33,6 +33,7 @@ class TaskState(Enum):
     executing = auto(), '🔥'
     removed = auto(), '☑️'
     canceled = auto(), '❌'
+    no_info = 99, '!'
 
     def __new__(cls, *args, **kwargs):
         value = len(cls.__members__) + 1
@@ -49,9 +50,13 @@ class TaskState(Enum):
 
 
 class BotInfo(BaseModel):
-    bot_token: Optional[str] = None
-    info_level: TaskState
+    bot_token: str
     chat_ids: List[int]
+    info_level: TaskState = TaskState.no_info
+    info_on_execution_start: bool = False
+    info_on_execution_end: bool = True
+    msg_on_success: str = None
+    msg_on_exception: str = None
 
 
 class AllowedTrigger(Enum):
@@ -65,8 +70,8 @@ class JobInfo(BaseModel):
     args: Optional[List] = []  # args to pass to the function
     kwargs: Optional[Dict] = {}  # kwargs to pass to the function
     pydantic_args: Optional[
-        List[tuple[str, Any]]] = None  # args tuple of [] will be first parsed by model.parse_obj
-    pydantic_kwargs: Optional[Dict[str, tuple[str, Any]]] = None  # kwargs to pass to the function
+        List[tuple[str, Any]]] = []  # args tuple of [] will be first parsed by model.parse_obj
+    pydantic_kwargs: Optional[Dict[str, tuple[str, Any]]] = {}  # kwargs to pass to the function
     trigger: Optional[AllowedTrigger] = AllowedTrigger.date  # the trigger object that controls the schedule of this job
     trigger_config: dict = dict()  # configuration of the triggers
     coalesce: bool = False  # whether to only run the job once when several run times are due
@@ -74,7 +79,7 @@ class JobInfo(BaseModel):
     # (None means “allow the job to run no matter how late it is”)
     max_instances: Optional[int] = 1
     run_in_background: Optional[bool] = False
-    bot_info: Optional[BotInfo]
+    bot_info: Optional[List[BotInfo]] = []
     error_msg: Optional[str]  # error msg
 
     def __str__(self):
@@ -109,8 +114,8 @@ class Task(BaseModel):
             self.id,
             self.job_info.module + '.' + self.job_info.callback,
             self.target_system,
-            self.job_info.args,
-            self.job_info.kwargs,
+            self.job_info.args + self.job_info.pydantic_args,
+            {**self.job_info.kwargs, **self.job_info.pydantic_kwargs},
             self.status.name,
             self.job
         )
@@ -121,8 +126,8 @@ class Task(BaseModel):
             self.status,
             self.target_system,
             self.job_info.module + '.' + self.job_info.callback,
-            self.job_info.args,
-            self.job_info.kwargs
+            {**self.job_info.kwargs, **self.job_info.pydantic_kwargs},
+            self.job_info.kwargs.update(self.job_info.pydantic_kwargs),
         )
 
     @classmethod
@@ -158,46 +163,53 @@ class Task(BaseModel):
         task.update(**kwarg)
 
     @staticmethod
-    def send_bot_info(task: "Task", msg: str):
+    def send_bot_info(bot_info: BotInfo, msg: str):
         try:
-            bot = Bot(task.job_info.bot_info.bot_token)
-            for chat_id in task.job_info.bot_info.chat_ids:
+            bot = Bot(bot_info.bot_token)
+            for chat_id in bot_info.chat_ids:
                 bot.send_message(chat_id, msg, parse_mode='HTML')
         except Exception as err:
             logger.error("could not send info from bot")
             logger.error(f"{type(err).__name__}: {err}")
 
     def execution_info_start(self, warning: str = None):
-        if self.job_info.bot_info is not None:
-            if self.status.value >= self.job_info.bot_info.info_level.value:
+        # send bot info
+        for bot_info in self.job_info.bot_info:
+            if bot_info.info_on_execution_start:
                 if warning is not None:
                     msg = f"⚠️ Warning\n<code>{warning}</code>\n\n"
                 else:
                     msg = f"📍 execution started\n"
                 msg += f"<code>{self.__repr__()}</code>"
-                Task.send_bot_info(self, msg)
+                self.send_bot_info(bot_info, msg)
 
     def execution_info_end(self, result: str = None, duration: timedelta = None):
         query = f"INSERT INTO db_global.Executions(task_id, result_msg, duration) VALUES (%s,%s,%s)"
         _ = execute_query(query, (self.id, result, str(duration)))
-        if self.job_info.bot_info is not None:
-            if self.status.value >= self.job_info.bot_info.info_level.value:
-                msg = f"✅ execution finished\n"
-                msg += f"<code>{self.__repr__()}</code>"
-                msg += f"\n\nresult:\n<code>{result}</code>"
-                msg += f"\n\nduration:\n<code>{duration}</code>"
-                Task.send_bot_info(self, msg)
+        for bot_info in self.job_info.bot_info:
+            if bot_info.info_on_execution_end:
+                if bot_info.msg_on_success is None:
+                    msg = f"✅ execution finished\n"
+                    msg += f"<code>{self.__repr__()}</code>"
+                    msg += f"\n\nresult:\n<code>{result}</code>"
+                    msg += f"\n\nduration:\n<code>{duration}</code>"
+                else:
+                    msg = bot_info.msg_on_success
+                self.send_bot_info(bot_info, msg)
 
     def execution_info_end_with_error(self, error: BaseException, duration: timedelta = None):
         query = f"INSERT INTO db_global.Executions(task_id, result_msg, duration) VALUES (%s,%s,%s)"
         error_msg = f"{type(error).__name__}: {error}"
         _ = execute_query(query, (self.id, error_msg, str(duration)))
-        if self.job_info.bot_info is not None:
-            if self.status.value >= self.job_info.bot_info.info_level.value:
-                msg = f"❌ execution finished with error\n"
-                msg += f"<code>{self.__repr__()}</code>"
-                msg += f"\n\nerror:\n<code>{error_msg}</code>"
-                Task.send_bot_info(self, msg)
+        for bot_info in self.job_info.bot_info:
+            if bot_info.info_on_execution_end:
+                if bot_info.msg_on_exception is None:
+                    msg = f"❌ execution finished with error\n"
+                    msg += f"<code>{self.__repr__()}</code>"
+                    msg += f"\n\nerror:\n<code>{error_msg}</code>"
+                else:
+                    msg = bot_info.msg_on_exception
+                self.send_bot_info(bot_info, msg)
 
     def update(self, **kwargs):
         if "status" in kwargs and kwargs["status"] != self.status:
@@ -205,10 +217,11 @@ class Task(BaseModel):
             query = f'UPDATE db_global.ScheduleJob SET status=%s WHERE id=%s'
             _ = execute_query(query, (self.status.value, self.id))
             # send bot info
-            if self.job_info.bot_info is not None and self.status.value >= self.job_info.bot_info.info_level.value:
-                msg = f"{self.status.symbol} Task has new status: {self.status.name}\n"
-                msg += f"<code>{self.__repr__()}</code>"
-                Task.send_bot_info(self, msg)
+            for bot_info in self.job_info.bot_info:
+                if self.status.value >= bot_info.info_level.value:
+                    msg = f"{self.status.symbol} Task has new status: {self.status.name}\n"
+                    msg += f"<code>{self.__repr__()}</code>"
+                    self.send_bot_info(bot_info, msg)
         if "job_info" in kwargs:
             self.job_info = kwargs["job_info"]
             query = f'UPDATE db_global.ScheduleJob SET job_info = %s WHERE id = %s'
@@ -368,10 +381,11 @@ class ScheduleThread(threading.Thread):
                 logger.warning("setting max instances to 1 since this is not a background job")
                 task.job_info.max_instances = 1
         # send bot info
-        if task.job_info.bot_info is not None:
-            msg = f"{task.status.symbol} Task created!\n"
-            msg += f"<code>{task.__repr__()}</code>"
-            Task.send_bot_info(task, msg)
+        for bot_info in task.job_info.bot_info:
+            if TaskState.new.value >= bot_info.info_level.value:
+                msg = f"{task.status.symbol} Task created!\n"
+                msg += f"<code>{task.__repr__()}</code>"
+                Task.send_bot_info(bot_info, msg)
         scheduler_job = self.scheduler.add_job(id=str(task.id),
                                                func=ScheduleThread.task_template,
                                                args=[task],
